`default_nettype	none
`timescale			1ns/1ps
/* -----------------------------------------------------------------
	AVALON Bus Async-SRAM Controller		TAB = 4;
	Verilog-HDL Version

	Synthesis & Fitter tool = "QuartusII Ver10.1+SP1"

	FileName = AsyncSRAM.v
	
	M.YOSHIDA				23.MAR.2011		REV	1.0
----------------------------------------------------------------- */
//	Define Declaration
//	Module Declaration

module	AsyncSRAM  #(	//parameter		ADRS_WIDTH			=	19				//	512KBytes
						parameter		ADRS_WIDTH			=	20				//	1024KBytes
						//parameter		ADRS_WIDTH			=	21				//	2048KBytes
					)
					(
						input	wire						clk,
						input	wire						reset,

						input	wire	[ADRS_WIDTH-1:0]	address,
						input	wire						chipselect,
						output	reg		[7:0]				readdata,
						input	wire	[7:0]				writedata,
						input	wire						write,
						input	wire						read,
						input	wire						byteenable,

						output	reg		[ADRS_WIDTH-1:0]	ab,
						inout	wire	[7:0]				db,
						output	reg							wen,
						output	reg							oen,
						output	reg							csn
					);

	reg		[7:0]				db_tmp;

	always	@( posedge clk or posedge reset )	begin
		if( reset )	begin
			ab	<=	{ ADRS_WIDTH { 1'b0 }};
		end
		else	begin
			ab	<=	address;
		end
	end

	always	@( posedge clk or posedge reset )	begin
		if( reset )	begin
			csn		<=	1'b1;
		end
		else	begin
			csn		<=	!( chipselect & ( read | write ));
		end
	end

	always	@( posedge clk or posedge reset )	begin
		if( reset )	begin
			oen		<=	1'b1;
		end
		else	begin
			oen		<=	!( chipselect & read );
		end
	end

	assign	db		=	db_tmp;
	always	@( posedge clk or posedge reset )	begin
		if( reset )	begin
			db_tmp		<=	8'hzz;
		end
		else	begin
			if( chipselect & write )	begin
				db_tmp	<=	writedata;
			end
			else	begin
				db_tmp	<=	8'hzz;
			end
		end
	end

	always	@( posedge clk or posedge reset )	begin
		if( reset )	begin
			wen		<=	1'b1;
		end
		else	begin
			wen		<=	!( chipselect & write );
		end
	end

	always	@( posedge clk or posedge reset )	begin
		if( reset )	begin
			readdata	<=	8'b0;
		end
		else	begin
			readdata	<=	db;
		end
	end

	/*	--------	DEBUG & DUMMY Signal Declaration	--------	*/

endmodule
