`default_nettype	none
`timescale			1ns/1ps
/* -----------------------------------------------------------------
	Smart-USB Plus System Controller		TAB = 4;
	Register Controller Verilog-HDL Version

	Synthesis & Fitter tool = "QuartusII11.1+SP1"

	FileName = GPIF_Mem.v
	
	M.YOSHIDA				17.FEB.2012		Ver	1.7
----------------------------------------------------------------- */
//	Define Declaration
//	Module Declaration
module	GPIF_Mem	(	clk,				//	System clock 48MHz input
						rstn,				//	Active low reset input
						data,				//	FX2 data bus input
						rdn,				//	Read enable signal active low
						wrn,				//	Write enable singnal active low
						cmdn,				//	Command enable signal active low
						rgdtn,				//	Register/Data indicate signal
						waitn,
						rfifo_q,

						mem_base_adrs,
						mem_adrs,			//	Current register address
						mem_write,
						mem_read,
						mem_exec,
						write_data,
						read_data,
						wait_request
					);

	parameter	REG_WRITE_WAIT = 0;
	
	
	parameter	WRITE_IDLE		=	3'b000;
	parameter	WRITE_READ1		=	3'b001;
	parameter	WRITE_READ2		=	3'b010;
	parameter	WRITE_EXEC		=	3'b011;
	parameter	WRITE_DONE		=	3'b100;

	parameter	READ_IDLE		=	3'b000;
	parameter	READ_WAIT1		=	3'b001;
	parameter	READ_WAIT2		=	3'b010;
	parameter	READ_LATCH		=	3'b011;
	parameter	READ_DONE		=	3'b100;

	//	port declaration
	input							clk, rstn;

	input	[15:0]					data;
	input							rdn;
	input							wrn;
	input							cmdn;
	input							rgdtn;
	output							waitn;
	output	[15:0]					rfifo_q;
	
	input	[31:0]					mem_base_adrs;
	output	[31:0]					mem_adrs;
	output							mem_write;
	output							mem_read;
	output							mem_exec;
	
	output	[31:0]					write_data;
	input	[31:0]					read_data;	
	input							wait_request;

	//	reg & wire declaration
	reg		[1:0]					mem_cmd;
	reg		[2:0]					length_cnt;
	reg		[3:0]					read_cnt;
	reg		[3:0]					write_cnt;
	reg								mem_exec;
	
	reg								mem_write_cmd;
	reg								mem_read_cmd;
	reg								write_cmd_rst;

	reg		[31:0]					mem_adrs_w;
	wire	[31:0]					mem_adrs;
	
	wire							wfifo_wreq;
	wire							wfifo_rreq;
	wire	[15:0]					wfifo_q;
	wire							wfifo_full;
	wire							wfifo_empty;
	wire	[11:0]					wfifo_usedw;

	wire							rfifo_wreq;
	wire							rfifo_rreq;
	wire	[15:0]					rfifo_q;
	wire							rfifo_full;
	reg		[31:0]					read_buff;
	
	reg		[2:0]					write_state;
	reg		[2:0]					write_next_state;
	reg		[31:0]					write_data;

	reg		[2:0]					read_state;
	reg		[2:0]					read_next_state;

	/*	--------	Memory Command Register Declaration	--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			//	reset state
			mem_cmd		<=	2'b00;
		end
		else if( !wrn & !cmdn & !rgdtn & ( write_cnt == REG_WRITE_WAIT ))	begin
			//	command latch state
			mem_cmd		<=	data[7:6];
		end
		else	begin
			//	command hold state
			mem_cmd		<=	mem_cmd;
		end
	end


	/*	--------	Write_Counter Declaration	--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			//	reset state
			write_cnt	<=	4'b0000;
		end
		else if( !wrn )	begin
			//	increment state
			write_cnt	<=	write_cnt + 1'b1;
		end
		else	begin
			//	reset state
			write_cnt	<=	4'b0000;
		end
	end


	/*	--------	Write State Machine	--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn ) begin
		//	reset state
			write_state	<=	WRITE_IDLE;
		end
		else begin
		//	next_state
			write_state	<=	write_next_state;
		end
	end

	always	@*	begin

		case( write_state )

			//	IDLE State
			WRITE_IDLE:		begin
								if(( wfifo_usedw >= 2 ) | wfifo_full )	begin
									write_next_state	<=	WRITE_READ1;
								end
								else	begin
									write_next_state	<=	WRITE_IDLE;
								end
							end

			//	READ1 State
			WRITE_READ1:	begin
								write_next_state	<=	WRITE_READ2;
							end

			//	READ2 State
			WRITE_READ2:	begin
								write_next_state	<=	WRITE_EXEC;
							end

			//	EXEC State
			WRITE_EXEC:		begin
								if( wait_request )	begin
									write_next_state	<=	WRITE_EXEC;
								end
								else	begin									
									write_next_state	<=	WRITE_DONE;
								end
							end

			//	DONE State
			WRITE_DONE:		begin
								if( wfifo_usedw >= 2 )	begin
									write_next_state	<=	WRITE_READ1;
								end
								else	begin
									write_next_state	<=	WRITE_IDLE;
								end
							end


		endcase
	end

	/*	--------	Read State Machine	--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn ) begin
		//	reset state
			read_state	<=	READ_IDLE;
		end
		else begin
		//	next_state
			read_state	<=	read_next_state;
		end
	end

	always	@*	begin

		case( read_state )

			//	IDLE State
			READ_IDLE:		begin
								if( !mem_read_cmd )	begin
									read_next_state	<=	READ_IDLE;
								end
								else	begin
									read_next_state	<=	READ_WAIT1;
								end
							end

			//	WAIT1 State
			READ_WAIT1:		begin
								if( rfifo_full )	begin
									read_next_state	<=	READ_WAIT1;
								end
								else	begin									
									read_next_state	<=	READ_WAIT2;
								end
							end

			//	WAIT2 State
			READ_WAIT2:		begin
								if( wait_request )	begin
									read_next_state	<=	READ_WAIT2;
								end
								else	begin									
									read_next_state	<=	READ_LATCH;
								end
							end

			//	LATCH State
			READ_LATCH:		begin
								read_next_state	<=	READ_DONE;
							end

			//	DONE State
			READ_DONE:		begin
								if( !mem_read_cmd )	begin
									read_next_state	<=	READ_IDLE;
								end
								else	begin
									read_next_state	<=	READ_WAIT1;
								end
							end
		endcase
	end


	/*	--------	Memory Write Command Detector Declaration		--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			mem_write_cmd	<=	1'b0;
		end
		else	begin
			if( !wrn & !cmdn & !rgdtn & ( data[1:0] == 2'b10 ))	begin
				mem_write_cmd	<=	1'b1;
			end
//			else if( !wrn & !cmdn & !rgdtn & ( data[1:0] == 2'b00 ))	begin						//	original
			else if( write_cmd_rst & ( wfifo_usedw <= 1 ) & ( write_state == WRITE_IDLE ))	begin			//	ADD		2011.03.22
				mem_write_cmd	<=	1'b0;
			end
			else	begin
				mem_write_cmd	<=	mem_write_cmd;
			end
		end
	end

	//	ADD		"write_cmd_rst" flag
	//	2011.03.22	
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			write_cmd_rst	<=	1'b0;
		end
		else	begin
			if( !wrn & !cmdn & !rgdtn & ( data[1:0] == 2'b00 ))	begin
				write_cmd_rst	<=	1'b1;
			end
			else if( !mem_write_cmd )	begin
				write_cmd_rst	<=	1'b0;
			end
			else	begin
				write_cmd_rst	<=	write_cmd_rst;
			end
		end
	end
	

	/*	--------	Memory Read Command Detector Declaration		--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			mem_read_cmd	<=	1'b0;
		end
		else	begin
			if( !wrn & !cmdn & !rgdtn & ( data[1:0] == 2'b11 ))	begin
				mem_read_cmd	<=	1'b1;
			end
			else if( !wrn & !cmdn & !rgdtn & ( data[1:0] == 2'b01 ))	begin
				mem_read_cmd	<=	1'b0;
			end
			else	begin
				mem_read_cmd	<=	mem_read_cmd;
			end
		end
	end

	/*	--------	Memory Command Executable Signal Declaration		--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			mem_exec	<=	1'b0;
		end
		else	begin
			mem_exec	<=	mem_read_cmd | mem_write_cmd;
		end
	end

	//	Write FIFO
	//	GPIF -> AVALON
	assign	wfifo_wreq	=	mem_write_cmd & !wrn & !rgdtn & cmdn;
	FIFOw		wfifo(		.clock( clk ),
							.aclr( !rstn | !mem_write_cmd ),
							.wrreq( wfifo_wreq ),
							.data( data ),
							.rdreq(( write_state == WRITE_READ1 ) | ( write_state == WRITE_READ2 )),
							.q( wfifo_q ),
							.empty( wfifo_empty ),
							.full( wfifo_full ),
							.usedw( wfifo_usedw )
					);


	//	Read FIFO
	//	GPIF <- AVALON
	FIFOr		rfifo(		.clock( clk ),
							.aclr( !rstn | !mem_read_cmd ),
							.wrreq(( read_state == READ_LATCH ) | ( read_state == READ_DONE )),
							.data(( read_state == READ_LATCH ) ? read_buff[15:0] : read_buff[31:16] ),
							.rdreq( mem_read_cmd & !rdn & !rgdtn & cmdn ),
							.q( rfifo_q ),
							.empty(),
							.full( rfifo_full ),
							.usedw()
					);


	/*	--------	Memory Write-Data Detector Declaration		--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			write_data	<=	32'h0000_0000;
		end
		else	begin
			if( write_state == WRITE_READ1 )	begin
				write_data[15:0]	<=	wfifo_q;
			end
			else	if( write_state == WRITE_READ2 )	begin
				write_data[31:16]	<=	wfifo_q;
			end
			else	begin
				write_data	<=	write_data;
			end
		end
	end


	/*	--------	Memory Address Detector Declaration		--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			mem_adrs_w	<=	32'h0000_0000;
		end
		else	begin
			if( mem_write_cmd | mem_read_cmd )	begin
				if(( write_state == WRITE_DONE ) | ( read_state == READ_DONE ))	begin
					//	2009.09.30	Add
					if( mem_base_adrs[31] )	begin
						mem_adrs_w	<=	mem_adrs_w;
					end
					else	begin
						mem_adrs_w	<=	mem_adrs_w + 1'b1;
					end
				end
				else	begin
					mem_adrs_w	<=	mem_adrs_w;
				end			
			end
			else	begin
				mem_adrs_w	<=	{ 2'b00, mem_base_adrs[31:2] };
			end
		end
	end

	/*	--------	Memory ReadData Buffer Declaration		--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			read_buff	<=	32'h0000_0000;
		end
		else	begin
//			if( read_state == READ_WAIT2 )	begin
			if(( read_state == READ_WAIT2 ) & !wait_request )	begin
				read_buff	<=	read_data;
			end
			else	begin
				read_buff	<=	read_buff;
			end
		end
	end

	/*	--------	Wire Declaration	--------	*/
	assign	mem_adrs	=	{ mem_adrs_w[29:0], 2'b00 }; 
	assign	mem_write	=	( write_state == WRITE_EXEC );
	assign	mem_read	=	( read_state == READ_WAIT2 );
	assign	waitn		=	!(( mem_read_cmd & !rfifo_full ) | ( mem_write_cmd & ( wfifo_full | ( wfifo_usedw > 2048 ))));

	/*	--------	Dummy Declaration	--------	*/

endmodule
