`default_nettype	none
`timescale			1ns/1ps
/* -----------------------------------------------------------------
	GPIF to AVALON bridge		TAB = 4;
	Verilog-HDL Version
	Synthesis & Fitter tool = "QuartusII11.1+SP2"

	FileName = GPIF_Master.v
	
	M.YOSHIDA				17.FEB.2012		Ver	1.7
----------------------------------------------------------------- */
//	Module Declaration
module	GPIF_Master	(		//	AVALON Master Port
							clk,
							reset,
							address,
							byteenable,
							read,
							write,
							writedata,
							readdata,
							waitrequest,

							//	External GPIF Port
							fd,					//	GPIF Bidirectional data bus
							ctl,				//	GPIF control signal
							rdy					//	GPIF status signal
					);

	parameter	REG_WRITE_WAIT = 0;
	parameter	REG_READ_WAIT = 0;
	parameter	REG_BASE_ADRS = 32'h0;

	//	port declaration
	//	AVALON Master Port
	input							clk;
	input							reset;
	output	[31:0]					address;
	output	[3:0]					byteenable;
	output							write;
	output	[31:0]					writedata;
	output							read;
	input	[31:0]					readdata;
	input							waitrequest;

	//	External GPIF Port
	inout	[15:0]					fd;
	input	[5:0]					ctl;
	output	[5:0]					rdy;

	wire	[63:0]					pbus_reg;
	wire							reg_latch;
	wire	[31:0]					reg_adrs;
	wire							reg_read;
	wire	[2:0]					reg_length;

	wire							rdn;
	wire							wrn;
	wire							cmdn;
	wire							rgdtn;
	wire							waitn;
	wire	[15:0]					rfifo_q;
	
	wire	[31:0]					mem_adrs;
	wire	[31:0]					mem_write_data;
	wire	[31:0]					mem_base_adrs;
	
	reg		[3:0]					read_cnt;
	reg		[3:0]					write_cnt;

	reg		[15:0]					data_tmp;
	reg		[15:0]					reg_sel;
	reg								rdn_delay;
	reg								rgdtn_delay;
	reg		[31:0]					reg_readdata;
	reg		[3:0]					byteenable;
	wire							mem_exec;

	wire							reg_write;
	wire							mem_write;
	wire							mem_read;

	assign	rdn			=	ctl[0];
	assign	wrn			=	ctl[1];
	assign	cmdn		=	ctl[2];
	assign	rgdtn		=	ctl[3];

	/*	--------	Data Bus Declaration	--------	*/
	assign	fd	=	data_tmp;
	always	@( posedge clk )	begin
		if(	rdn_delay )	begin
			data_tmp	<=	16'hzzzz;
		end
		else if( rgdtn_delay )	begin
			data_tmp	<=	reg_sel;
		end
		else	begin
			data_tmp	<=	rfifo_q;
		end
	end

	always	@( posedge clk )	begin

		rdn_delay	<=	rdn;
		rgdtn_delay	<=	rgdtn;
			
	end

	

	/*	--------	AVALON Register-Read Data Declaration	--------	*/
	always	@( posedge clk or posedge reset )	begin
		if( reset )	begin
			//	reset state
			reg_readdata	<=	32'h0000_0000;
		end
		else if( reg_read )	begin
			//	command latch state
			if( reg_adrs == REG_BASE_ADRS )	begin
				reg_readdata	<=	mem_base_adrs;
			end
			else	begin
				reg_readdata	<=	readdata;
			end
		end
		else	begin
			//	command hold state
			reg_readdata	<=	reg_readdata;
		end
	end

	/*	--------	AVALON Byte-Enable Signal Declaration	--------	*/
	always	@( posedge clk or posedge reset )	begin
		if( reset )	begin
			//	reset state
			byteenable	<=	4'b0000;
		end
		else	begin
			if( mem_exec )	begin
				byteenable	<=	4'b1111;
			end
			else	begin
				if( !wrn & !cmdn & ( write_cnt == REG_WRITE_WAIT ))	begin
					case( fd[7:6] )
						2'b00	:	byteenable	<=	4'b0001;		//	8bit Command
						2'b01	:	byteenable	<=	4'b0011;		//	16bit Command
						2'b10	:	byteenable	<=	4'b1111;		//	32bit Command
						2'b11	:	byteenable	<=	4'b1111;		//	64bit Command
						default	:	byteenable	<=	4'b0000;
					endcase
				end
				else	begin
					byteenable	<=	byteenable;
				end
			end
		end
	end

	/*	--------	Read_Counter Declaration	--------	*/
	always	@( posedge clk or posedge reset )	begin
		if( reset )	begin
			//	reset state
			read_cnt	<=	4'b0000;
		end
		else if( !rdn )	begin
			//	increment state
			read_cnt	<=	read_cnt + 1'b1;
		end
		else	begin
			//	reset state
			read_cnt	<=	4'b0000;
		end
	end

	/*	--------	Write_Counter Declaration	--------	*/
	always	@( posedge clk or posedge reset )	begin
		if( reset )	begin
			//	reset state
			write_cnt	<=	4'b0000;
		end
		else if( !wrn )	begin
			//	increment state
			write_cnt	<=	write_cnt + 1'b1;
		end
		else	begin
			//	reset state
			write_cnt	<=	4'b0000;
		end
	end




	/*	------------	32Bit to 16Bit selector		------------	*/
	always 	@*	begin
		case( reg_length[1:0] )
			2'b00	:	reg_sel	<=	reg_readdata[15:0];
			2'b01	:	reg_sel	<=	reg_readdata[31:16];
			default	:	reg_sel	<=	16'h0000;
		endcase
	end


	/*	--------	GPIF Register Controller Declaration	--------	*/
	defparam	RegCont.REG_BASE_ADRS = REG_BASE_ADRS,
				RegCont.REG_READ_WAIT = REG_READ_WAIT,
				RegCont.REG_WRITE_WAIT = REG_WRITE_WAIT;
	GPIF_Reg	RegCont		(	.clk( clk ),					//	System clock 48MHz input
								.rstn( !reset ),				//	Active low reset input
								.data( fd ),					//	FX2 data bus input
								.rdn( ctl[0] ),					//	Read enable signal active low
								.wrn( ctl[1] ),					//	Write enable singnal active low
								.cmdn( ctl[2] ),				//	Command enable signal active low
								.rgdtn( ctl[3] ),				//	Register/Data indicate signal
								.pbus_reg( pbus_reg ),			//	Parallel data bus for Register
								.length_cnt( reg_length ),		//	Current length
								.reg_adrs( reg_adrs ),			//	Current register address
								.reg_write( reg_write ),
								.reg_read( reg_read ),
								.mem_base_adrs( mem_base_adrs ),
								.wait_request( waitrequest )
							);

	/*	--------	GPIF Memory Controller Declaration	--------	*/
	GPIF_Mem	MemCont		(	.clk( clk ),					//	System clock 48MHz input
								.rstn( !reset ),				//	Active low reset input
								.data( fd ),					//	FX2 data bus input
								.rdn( ctl[0] ),					//	Read enable signal active low
								.wrn( ctl[1] ),					//	Write enable singnal active low
								.cmdn( ctl[2] ),				//	Command enable signal active low
								.rgdtn( ctl[3] ),				//	Register/Data indicate signal
								.waitn( waitn ),
								.rfifo_q( rfifo_q ),
								.mem_base_adrs( mem_base_adrs ),
								.mem_adrs( mem_adrs ),			//	Current register address
								.mem_write( mem_write ),
								.mem_exec( mem_exec ),
								.mem_read( mem_read ),
								.write_data( mem_write_data ),
								.read_data( readdata ),
								.wait_request( waitrequest )
							);


	/*	--------	GPIF Status Signal Declaration	--------	*/
	assign	address		=	( mem_exec ) ? mem_adrs : reg_adrs;
	assign	writedata	=	( mem_exec ) ? mem_write_data : pbus_reg[31:0];

	assign	write		=	reg_write | mem_write;
	assign	read		=	reg_read | mem_read;

	assign	rdy[0]	=	waitn;
	assign	rdy[1]	=	1'b1;
	assign	rdy[2]	=	1'b1;
	assign	rdy[3]	=	1'b1;
	assign	rdy[4]	=	1'b1;
	assign	rdy[5]	=	1'b1;

endmodule
