`default_nettype	none
`timescale			1ns/1ps
/* -----------------------------------------------------------------
	Smart-USB Plus System Controller		TAB = 4;
	Register Controller Verilog-HDL Version

	Synthesis & Fitter tool = "QuartusII11.1+SP2"

	FileName = GPIF_Reg.v
	
	M.YOSHIDA				16.FEB.2012		Ver	1.7
----------------------------------------------------------------- */
//	Define Declaration
//	Module Declaration
module	GPIF_Reg	#(	parameter	REG_WRITE_WAIT	=	0,
						parameter	REG_READ_WAIT	=	0,
						parameter	REG_BASE_ADRS	=	32'h0
					)
					(	input	wire					clk,			//	System clock 48MHz input
						input	wire					rstn,			//	Active low reset input
						input	wire	[15:0]			data,			//	FX2 data bus input
						input	wire					rdn,			//	Read enable signal active low
						input	wire					wrn,			//	Write enable singnal active low
						input	wire					cmdn,			//	Command enable signal active low
						input	wire					rgdtn,			//	Register/Data indicate signal
						output	reg		[63:0]			pbus_reg,		//	Parallel data bus for Register
						output	reg		[2:0]			length_cnt,		//	Current length
						output	wire	[31:0]			reg_adrs,		//	Current register address
						output	reg						reg_write,
						output	reg						reg_read,
						output	reg		[31:0]			mem_base_adrs,
						input	wire					wait_request
					);

	
	localparam	IDLE			=	2'b00;
	localparam	EXEC			=	2'b01;
	localparam	WAIT			=	2'b10;

	//	reg & wire declaration
	reg		[15:0]					reg_cmd;
	reg								reg_latch;
	reg		[3:0]					read_cnt;
	reg		[3:0]					write_cnt;


	/*	--------	Register Command Register Declaration	--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			//	reset state
			reg_cmd		<=	8'h00;
		end
		else if( !wrn & !cmdn & rgdtn & ( write_cnt == REG_WRITE_WAIT ))	begin
			//	command latch state
			reg_cmd		<=	data[15:0];
		end
		else	begin
			//	command hold state
			reg_cmd		<=	reg_cmd;
		end
	end

	/*	--------	Length Counter Declaration	--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			//	reset state
			length_cnt	<=	2'b00;
		end
		else if( !wrn & !cmdn )	begin
			//	command recive reset state
			length_cnt	<=	2'b00;
		end
		else if(( !wrn & ( write_cnt == REG_WRITE_WAIT )) | ( read_cnt == REG_READ_WAIT + 1 ))	begin
			//	Write&Read state inclrement
			length_cnt	<=	length_cnt + 1'b1;
		end
		else	begin
			//	hold state
			length_cnt	<=	length_cnt;
		end
	end

	/*	--------	Register Latch Register Declaration	--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			//	reset state
			reg_latch	<=	1'b0;
		end
		else if( !wrn & cmdn & rgdtn & ( write_cnt == REG_WRITE_WAIT ))	begin
			//	latch state
			case( reg_cmd[7:6] )

				2'b00:	reg_latch	<=	( length_cnt == 0 );	//	8BIT
				2'b01:	reg_latch	<=	( length_cnt == 0 );	//	16BIT
				2'b10:	reg_latch	<=	( length_cnt == 1 );	//	32BIT
				2'b11:	reg_latch	<=	( length_cnt == 3 );	//	64BIT

			endcase
		end
		else	begin
			//	reset state
			reg_latch	<=	1'b0;
		end
	end

	/*	--------	Read_Counter Declaration	--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			//	reset state
			read_cnt	<=	4'b0000;
		end
		else if( !rdn )	begin
			//	increment state
			read_cnt	<=	read_cnt + 1'b1;
		end
		else	begin
			//	reset state
			read_cnt	<=	4'b0000;
		end
	end

	/*	--------	Write_Counter Declaration	--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			//	reset state
			write_cnt	<=	4'b0000;
		end
		else if( !wrn )	begin
			//	increment state
			write_cnt	<=	write_cnt + 1'b1;
		end
		else	begin
			//	reset state
			write_cnt	<=	4'b0000;
		end
	end

	/*	--------	Internal Parallel Data Bus Declaration	--------	*/
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			//	reset state
			pbus_reg	<=	64'h0000_0000_0000_0000;
		end
		else	begin
			if(( !wrn & ( write_cnt == REG_WRITE_WAIT )))	begin
				//	fd to pbus latch state
				case( length_cnt )

					2'b00	:	pbus_reg	<=	{ pbus_reg[63:16], data };					//	fd[15:0] Write to pbus[15:0]
					2'b01	:	pbus_reg	<=	{ pbus_reg[63:32], data, pbus_reg[15:0] };	//	fd[15:0] Write to pbus[31:16]
					2'b10	:	pbus_reg	<=	{ pbus_reg[63:48], data, pbus_reg[31:0] };	//	fd[15:0] Write to pbus[47:32]
					2'b11	:	pbus_reg	<=	{ data, pbus_reg[47:0] };					//	fd[15:0] Write to pbus[63:48]

				endcase
			end
			else	begin
				//	hold state
				pbus_reg	<=	pbus_reg;
			end
		end
	end


	always	@( posedge clk or negedge rstn )	begin
		if( !rstn ) begin
			reg_write	<=	1'b0;
		end
		else begin
			reg_write	<=	( reg_write & wait_request ) | reg_latch;
		end
	end

	always	@( posedge clk or negedge rstn )	begin
		if( !rstn ) begin
		//	reset state
			reg_read	<=	1'b0;
		end
		else begin
		//	next_state
			reg_read	<=	( reg_read & wait_request ) | ( !wrn & !cmdn & rgdtn );
		end
	end


	//	GPIF Memory BASE-Address Register Declaration
	//	GPIF Register0
	always	@( posedge clk or negedge rstn )	begin
		if( !rstn )	begin
			//	reset state
			mem_base_adrs	<=	32'h0;
		end
		else if( reg_write & ({ reg_cmd[15:8], reg_cmd[5:0] } == 14'd0 ))	begin
			//	increment state
			mem_base_adrs	<=	{ pbus_reg[31:2], 2'b00 };
		end
		else	begin
			//	reset state
			mem_base_adrs	<=	mem_base_adrs;
		end
	end


	/*	--------	Wire Declaration	--------	*/
//	assign	reg_adrs	=	REG_BASE_ADRS[29:0] + ( reg_cmd[5:0] << 2 ); 		//	Ver1.6
	assign	reg_adrs	=	REG_BASE_ADRS[29:0] + ( { reg_cmd[15:8], reg_cmd[5:0] } << 2 ); 

	/*	--------	Dummy Declaration	--------	*/

endmodule
